==> apache-doris-1.2.7.1-src.tar.gz: apache-doris-1.2.7.1-src/fe/fe-core/src/main/java/org/apache/doris/nereids/trees/expressions/functions/ExpressionTrait.java <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package org.apache.doris.nereids.trees.expressions.functions;

import org.apache.doris.nereids.exceptions.UnboundException;
import org.apache.doris.nereids.trees.TreeNode;
import org.apache.doris.nereids.trees.expressions.Expression;
import org.apache.doris.nereids.types.DataType;

import java.util.List;

/**
 * ExpressionTrait.
 */
public interface ExpressionTrait extends TreeNode<Expression> {

    boolean nullable();

    default boolean notNullable() {
        return !nullable();
    }

    default List<Expression> getArguments() {
        return children();
    }

    default Expression getArgument(int index) {
        return child(index);
    }

    default DataType getDataType() throws UnboundException {
        throw new UnboundException("dataType");
    }

    default String toSql() throws UnboundException {
        throw new UnboundException("sql");
    }
}

==> apache-doris-2.0.0-src.tar.gz: apache-doris-2.0.0-src/fe/fe-core/src/main/java/org/apache/doris/nereids/trees/expressions/functions/ExpressionTrait.java <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package org.apache.doris.nereids.trees.expressions.functions;

import org.apache.doris.nereids.annotation.Developing;
import org.apache.doris.nereids.exceptions.UnboundException;
import org.apache.doris.nereids.trees.TreeNode;
import org.apache.doris.nereids.trees.expressions.Expression;
import org.apache.doris.nereids.types.DataType;

import com.google.common.collect.ImmutableList;

import java.util.List;

/**
 * ExpressionTrait.
 */
public interface ExpressionTrait extends TreeNode<Expression> {

    boolean nullable();

    default boolean notNullable() {
        return !nullable();
    }

    // check legality before do type coercion.
    // maybe we should merge checkInputDataTypes and checkLegality later.
    @Developing
    default void checkLegalityBeforeTypeCoercion() {}

    @Developing
    default void checkLegalityAfterRewrite() {}

    default List<Expression> getArguments() {
        return children();
    }

    default Expression getArgument(int index) {
        return child(index);
    }

    default List<DataType> getArgumentsTypes() {
        return getArguments()
                .stream()
                .map(Expression::getDataType)
                .collect(ImmutableList.toImmutableList());
    }

    default DataType getArgumentType(int index) {
        return child(index).getDataType();
    }

    default DataType getDataType() throws UnboundException {
        throw new UnboundException("dataType");
    }

    default String toSql() throws UnboundException {
        throw new UnboundException("sql");
    }

    default boolean foldable() {
        return true;
    }
}
